#include <stdio.h>
#include <assert.h>
#include <errno.h>
#include <unistd.h>
#include <sys/time.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netdb.h>

#include <arpa/inet.h>

/*
 * Skeleton client program. 
 *
 * It takes one argument: the name of machine on which the server is running.
 *
 * It takes this name, looks the socket address for it and then connects to
 * the server.
 *
 * Written by Frans Kaashoek (kaashoek@mit.edu)
 */


#include "server.h"

static char 			*progname;
static char			*server_host_name; 	/* server name */
static char			server_can_host_name[64]; /* canonical form */
static struct sockaddr_in	server_in; 		/* server socket addr */


/* 
 * Resolve name.
 * stolen from ping.c 
 */
static u_long resolve_name( namep, canonp, canonl )
    char *namep, canonp[] ;
    int  canonl ;
{
    struct hostent *hp ;
    u_long inetaddr ;
    int n ;
    
    if (isdigit(*namep)) {
	/* Assume dotted-decimal */
        inetaddr = (u_long) inet_addr(namep) ;
        *canonp = '\0' ;   /* No canonical name */
        return(inetaddr) ;
    }
    else  {
        if (NULL == (hp =  gethostbyname( namep )))
	    return(0) ;

        n = ((n = strlen(hp->h_name)) >= canonl) ?  canonl-1 : n ;
        memcpy(canonp, hp->h_name, n) ;
        canonp[n] = '\0' ;

        return( *( u_long *) hp->h_addr) ;
    }
}


/*
 * Send HELLO to server
 */
static void
client(int s)
{
#define HELLO	"hello world\n"

    /* Write a message on the connection. */
    if (write(s, HELLO, strlen(HELLO) + 1) < 0) {
	perror("client: write failed\n");
	exit(1);
    }
}


int main(int argc, char *argv[])
{
    int s;

    progname = argv[0];
    if (argc < 2) {
	printf("%s: %s server\n", progname, progname);
	exit(1);
    }

    server_host_name = argv[1];

    /* Resolve name of server name into a socket address. */
    server_in.sin_family = AF_INET;
    if ((server_in.sin_addr.s_addr = resolve_name(server_host_name,
					       server_can_host_name, 64)) == 0)
    { 
	printf("%s: unknown host %s\n", progname, server_host_name);
	exit(1);
    }

    /* Fill in the server port. */
    server_in.sin_port = htons(SERVER_PORT);
    
    /* Create a socket of type stream (instead of datagram). */
    if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	perror("main: socket failed");
	exit(1);
    }

    /* Set up a connection with the server. */
    if (connect(s, (struct sockaddr *) &server_in, sizeof(server_in)) < 0)
    {
	perror("main: connect failed");
	exit(1);
    }

    /* Now, do something useful. */
    client(s);
    
    /* Close connection. */
    close(s);

    return (0);
}

