#include <stdio.h>
#include <errno.h>
#include <sys/time.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <netdb.h>

#include "server.h"

/*
 * Skeleton server program.
 * It takes no arguments.
 *
 * Server program loops forever listening on port SERVER_PORT waiting
 * for client to connect. Once a client connects the program forks a
 * a copy of itself to deal with the client, while the orginal programs
 * keeps waiting for other clients.
 *
 * Written by Frans Kaashoek (kaashoek@mit.edu).
 *
 */

#define MAXMSG			1024
#define MAXLISTEN		5

static char 			*progname;
static char			msg[MAXMSG];


/*
 * Server reads from the connection and prints what it receives.
 * Assumption: server can print what it reads from connection.
 */
static void 
server(int s)
{
    int len;

    if ((len = read(s, msg, MAXMSG)) < 0) {
	perror("read");
	exit(1);
    }
    
    printf("%s", msg);
}


int
main(int argc, char *argv[])
{
    int s;
    struct sockaddr_in from;
    static struct sockaddr_in me_in;
    int fromlen;
    int news;
    int pid;

    progname = argv[0];

    /* Create socket. */
    if ((s = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
	perror("socket");
	exit(1);
    }

    /* Bind my socket address to a connection. */
    me_in.sin_family = AF_INET;
    me_in.sin_addr.s_addr = htonl(INADDR_ANY); /* anyone can connect */
    me_in.sin_port = htons(SERVER_PORT);
    if (bind(s, (struct sockaddr *) &me_in, sizeof(me_in)) < 0) {
	perror("bind");
	exit(1);
    }

    /* Server accepts MAXLISTEN connection requests at the same time. */
    if (listen(s, MAXLISTEN) < 0) {
	perror("listen");
	exit(1);
    }

    /* Loop forever accepting connections from anyone. */
    for (;;) {
	
	/* Wait for some to connect. Accepts returns a new connection
	 * for the established connection.
	 */
	fromlen = sizeof(from);
	if ((news = accept(s, (struct sockaddr *) &from, &fromlen)) < 0) {
	    perror("accept");
	    exit(1);
	}

	/* Some client connected; fork off a process to deal with client. 
	 * The main program can continue waiting for new connection.
	 */
	if((pid = fork()) < 0) {
	    perror("fork");
	    exit(1);
	} else if (pid == 0) {	/* child? */
	    close(s);		/* child has no need for the parent's socket */
	    server(news);	/* serve the established connection */
	    exit(0);
	}

	close(news);
    }
}
